-- *************************************************************************
--
-- Design of the SLS TriState Components Interface Module
--
-- Original Design (in Verilog)
--      Design by: Bhavesh Desai
--           Date: October 25, 2004
--        Version: 1.0
--
-- Modified Design (in VHDL)
--      Design by: Tyson Hall
--           Date: May 27, 2005
--        Version: 1.0
--
-- *************************************************************************


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity up3_bus_mux is
  port ( sram_be_n       : in  std_logic_vector(1 downto 0);
         lcd_cs_n        : in  std_logic;
         tri_addr_in     : in  std_logic_vector(20 downto 0);
         tri_be_n        : in  std_logic_vector(1 downto 0);
         flash_byte_n    : in  std_logic;
         tri_read_n      : in  std_logic;
         lcd_read_n      : in  std_logic;
         sram_read_n     : in  std_logic;
         flash_cs_n      : in  std_logic;
         sram_cs_n       : in  std_logic;
         lcd_we_n        : in  std_logic;  
         flash_we_n      : in  std_logic;
         sram_we_n       : in  std_logic;
         sdram_ba        : in  std_logic_vector(1 downto 0);
         sdram_cas_n     : in  std_logic;
         sdram_cke       : in  std_logic;
         sdram_cs_n      : in  std_logic;
         sdram_ras_n     : in  std_logic;
         sdram_we_n      : in  std_logic;
         fl_sr_rd_n      : out std_logic;
         fl_sr_sd_we_n   : out std_logic;
         tri_addr        : out std_logic_vector(19 downto 0);
         fl_byte_sd_cke  : out std_logic;
         lcd_rw_n        : out std_logic;
         lcd_rs          : out std_logic;
         lcd_en          : out std_logic         );
end up3_bus_mux;

architecture arch of up3_bus_mux is

begin

  lcd_rw_n <= not(lcd_cs_n) and tri_addr_in(0);
  lcd_rs   <= not(lcd_cs_n) and tri_addr_in(1);
  lcd_en   <= not(lcd_we_n) or not(lcd_read_n);

  tri_addr(19 downto 18) <= tri_addr_in(20 downto 19);
  tri_addr(11 downto 0)  <= tri_addr_in(12 downto 1) when (sdram_cs_n = '1')
                           else tri_addr_in(11 downto 0);
    
  process( flash_cs_n,  sram_cs_n, sdram_cs_n, tri_addr_in(18 downto 13), tri_be_n,
           tri_read_n, flash_we_n, flash_byte_n, sram_read_n, sram_we_n, sram_be_n,
           sdram_we_n, sdram_ba, sdram_cas_n, sdram_ras_n, sdram_cke )
  begin

    if (flash_cs_n = '0') then
      fl_sr_rd_n             <= tri_read_n;
      fl_sr_sd_we_n          <= flash_we_n;
      tri_addr(17 downto 12) <= tri_addr_in(18 downto 13);
      fl_byte_sd_cke         <= flash_byte_n;
      
    elsif (sram_cs_n = '0') then
      fl_sr_rd_n             <= sram_read_n;
      fl_sr_sd_we_n          <= sram_we_n;
      tri_addr(17 downto 12) <= sram_be_n(1 downto 0) & tri_addr_in(16 downto 13);
      fl_byte_sd_cke         <= flash_byte_n;
      
    elsif (sdram_cs_n = '0') then
      fl_sr_rd_n             <= '1';
      fl_sr_sd_we_n          <= sdram_we_n;
      tri_addr(17 downto 12) <= tri_be_n(1 downto 0) & sdram_ras_n &
                                sdram_cas_n & sdram_ba(1 downto 0);
      fl_byte_sd_cke         <= sdram_cke;
      
    else
      fl_sr_rd_n             <= '1';
      fl_sr_sd_we_n          <= '1';
      tri_addr(17 downto 12) <= "111111";
      fl_byte_sd_cke         <= '1';

    end if;
    
  end process;

end arch;
