-- This file shows an example of using the up3core package in VHDL
-- This saves time since core component declarations are made in the package
LIBRARY IEEE;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

PACKAGE up3core IS
	COMPONENT LCD_Display
	GENERIC (Num_Hex_Digits: Integer);
	PORT(Hex_Display_Data: 	IN    STD_LOGIC_VECTOR
						((Num_Hex_Digits*4)-1 DOWNTO 0);
		 reset, clk_48Mhz:	 IN	STD_LOGIC;
		 LCD_RS, LCD_E: 	OUT	STD_LOGIC;
		 LCD_RW: 		INOUT STD_LOGIC;
	DATA_BUS: 		INOUT	STD_LOGIC_VECTOR(7 DOWNTO 0));
	END COMPONENT;
	COMPONENT debounce
		PORT(pb, clock_100Hz 		: IN	STD_LOGIC;
	         pb_debounced			: OUT	STD_LOGIC);
	END COMPONENT;
	COMPONENT onepulse
		PORT(pb_debounced, clock	: IN	STD_LOGIC;
		 	pb_single_pulse			: OUT	STD_LOGIC);
	END COMPONENT;
	COMPONENT clk_div
		PORT(clock_48Mhz			: IN	STD_LOGIC;
			 clock_1MHz				: OUT	STD_LOGIC;
			 clock_100KHz			: OUT	STD_LOGIC;
			 clock_10KHz			: OUT	STD_LOGIC;
			 clock_1KHz				: OUT	STD_LOGIC;
			 clock_100Hz			: OUT	STD_LOGIC;
			 clock_10Hz				: OUT	STD_LOGIC;
			 clock_1Hz				: OUT	STD_LOGIC);
	END COMPONENT;
	COMPONENT vga_sync
 		PORT(clock_48Mhz, red, green, blue		: IN	STD_LOGIC;
         	 red_out, green_out, blue_out 		: OUT 	STD_LOGIC;
			 horiz_sync_out, vert_sync_out		: OUT 	STD_LOGIC;
			 pixel_row, pixel_column	: OUT STD_LOGIC_VECTOR(9 DOWNTO 0));
	END COMPONENT;
	COMPONENT Char_ROM
		PORT(clock					: IN 	STD_LOGIC;
			 character_address		: IN	STD_LOGIC_VECTOR(5 DOWNTO 0);
			 font_row, font_col		: IN 	STD_LOGIC_VECTOR(2 DOWNTO 0);
			 rom_mux_output			: OUT	STD_LOGIC);
	END COMPONENT;
	COMPONENT keyboard
		PORT(keyboard_clk, keyboard_data, clock_48Mhz , 
			 reset, read				: IN STD_LOGIC;
			 scan_code					: OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
			 scan_ready					: OUT	STD_LOGIC);
	END COMPONENT;
	COMPONENT mouse
		PORT(clock_48Mhz, reset 		: IN std_logic;
        	 mouse_data					: INOUT std_logic;
        	 mouse_clk 					: INOUT std_logic;
        	 left_button, right_button 	: OUT std_logic;
        	 mouse_cursor_row, 
			 mouse_cursor_column 		: OUT std_logic_vector(9 DOWNTO 0));       
		END COMPONENT;
END up3core;

LIBRARY work;
USE work.up3core.all;

LIBRARY IEEE;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;
-- Example showing use of up3core modify lines below for your design
ENTITY up3pack IS
	PORT (clock_48Mhz, pb1 : IN STD_LOGIC;
		  pb1_debounced : OUT STD_LOGIC);
END up3pack;

ARCHITECTURE a OF up3pack IS
SIGNAL clock_100Hz : STD_LOGIC;
BEGIN

	debounce1 : debounce PORT MAP  (pb => pb1, 
									clock_100Hz => clock_100Hz, 
									pb_debounced => pb1_debounced);

	prescalar : clk_div  PORT MAP (clock_48Mhz => clock_48Mhz,
									clock_100hz => clock_100hz);
END a;
		  


